@@ -37,6 +37,11 @@ module Happstack.Authenticate.Core
     , isAuthAdmin
     , usernameAcceptable
     , requireEmail
+    , systemFromAddress
+    , systemReplyToAddress
+    , systemSendmailPath
+    , postLoginRedirect
+    , createUserCallback
     , HappstackAuthenticateI18N(..)
     , UserId(..)
     , unUserId
@@ -88,6 +93,8 @@ module Happstack.Authenticate.Core
     , GetUserByUsername(..)
     , GetUserByUserId(..)
     , GetUserByEmail(..)
+    , GetUsers(..)
+    , GetUsersByEmail(..)
     , GetAuthenticateState(..)
     , getOrGenSharedSecret
     , Token(..)
@@ -161,6 +168,7 @@ import System.Random                   (randomRIO)
 import Text.Boomerang.TH               (makeBoomerangs)
 import Text.Shakespeare.I18N           (RenderMessage(renderMessage), mkMessageFor)
 import Web.JWT                         (Algorithm(HS256), JWT, VerifiedJWT, JWTClaimsSet(..), encodeSigned, claims, decode, decodeAndVerifySignature, secondsSinceEpoch, intDate, verify)
+import qualified Web.JWT               as JWT
 #if MIN_VERSION_jwt(0,8,0)
 import Web.JWT                         (ClaimsMap(..), hmacSecret)
 #else
@@ -362,6 +370,8 @@ data AuthenticateConfig = AuthenticateConfig
     , _systemFromAddress    :: Maybe SimpleAddress         -- ^ From: line for emails sent by the server
     , _systemReplyToAddress :: Maybe SimpleAddress         -- ^ Reply-To: line for emails sent by the server
     , _systemSendmailPath   :: Maybe FilePath              -- ^ path to sendmail if it is not \/usr\/sbin\/sendmail
+    , _postLoginRedirect    :: Maybe Text                  -- ^ path to redirect to after a successful login
+    , _createUserCallback   :: Maybe (User -> IO ())       -- ^ a function to call when a new user is created. Useful for adding them to mailing lists or other stuff
     }
     deriving (Typeable, Generic)
 makeLenses ''AuthenticateConfig
@@ -575,6 +585,13 @@ getUserByUserId userId =
     do us <- view users
        return $ getOne $ us @= userId
 
+-- | find all 'Users'
+--
+getUsers :: Query AuthenticateState (Set User)
+getUsers =
+    do us <- view users
+       return $ toSet $ us
+
 -- | look up a 'User' by their 'Email'
 --
 -- NOTE: if the email is associated with more than one account this will return 'Nothing'
@@ -584,6 +601,14 @@ getUserByEmail email =
     do us <- view users
        return $ getOne $ us @= email
 
+-- | find all 'Users' which match 'Email'
+--
+getUsersByEmail :: Email
+               -> Query AuthenticateState (Set User)
+getUsersByEmail email =
+    do us <- view users
+       return $ toSet $ us @= email
+
 -- | get the entire AuthenticateState value
 getAuthenticateState :: Query AuthenticateState AuthenticateState
 getAuthenticateState = ask
@@ -601,7 +626,9 @@ makeAcidic ''AuthenticateState
     , 'deleteUser
     , 'getUserByUsername
     , 'getUserByUserId
+    , 'getUsers
     , 'getUserByEmail
+    , 'getUsersByEmail
     , 'getAuthenticateState
     ]
 
@@ -708,7 +735,9 @@ decodeAndVerifyToken authenticateState now token =
                       Nothing -> return Nothing
                       (Just ssecret) ->
                         -- finally we can verify all the claims
-#if MIN_VERSION_jwt(0,8,0)
+#if MIN_VERSION_jwt(0,11,0)
+                        case verify (JWT.toVerify $ hmacSecret (_unSharedSecret ssecret)) unverified of
+#elif MIN_VERSION_jwt(0,8,0)
                         case verify (hmacSecret (_unSharedSecret ssecret)) unverified of
 #else
                         case verify (secret (_unSharedSecret ssecret)) unverified of
