@@ -1,22 +1,27 @@
 {-# LANGUAGE QuasiQuotes #-}
 module Happstack.Authenticate.Password.Controllers where
 
+import Control.Concurrent.STM               (atomically)
+import Control.Concurrent.STM.TVar          (TVar, readTVar)
+import Control.Monad.Trans                  (MonadIO(liftIO))
+import Data.Maybe                           (isJust, fromJust)
 import Data.Text                            (Text)
 import qualified Data.Text                  as T
-import Happstack.Authenticate.Core          (AuthenticateURL)
+import Happstack.Authenticate.Core          (AuthenticateConfig(_postLoginRedirect), AuthenticateURL)
 import Happstack.Authenticate.Password.URL (PasswordURL(Account, Token, Partial, PasswordReset, PasswordRequestReset), nestPasswordURL)
 import Happstack.Authenticate.Password.PartialsURL (PartialURL(ChangePassword, Logout, Login, LoginInline, SignupPassword, ResetPasswordForm, RequestResetPasswordForm))
 import Language.Javascript.JMacro
 import Web.Routes
 
-usernamePasswordCtrl :: (Monad m) => RouteT AuthenticateURL m JStat
-usernamePasswordCtrl =
+usernamePasswordCtrl :: (MonadIO m) => TVar AuthenticateConfig -> RouteT AuthenticateURL m JStat
+usernamePasswordCtrl authenticateConfigTV =
   nestPasswordURL $
     do fn <- askRouteFn
-       return $ usernamePasswordCtrlJs fn
+       plr <- liftIO (_postLoginRedirect <$> (atomically $ readTVar authenticateConfigTV))
+       return $ usernamePasswordCtrlJs plr fn
 
-usernamePasswordCtrlJs :: (PasswordURL -> [(Text, Maybe Text)] -> Text) -> JStat
-usernamePasswordCtrlJs showURL = [jmacro|
+usernamePasswordCtrlJs :: Maybe Text -> (PasswordURL -> [(Text, Maybe Text)] -> Text) -> JStat
+usernamePasswordCtrlJs postLoginRedirect showURL = [jmacro|
   {
     var usernamePassword = angular.module('usernamePassword', ['happstackAuthentication']);
 
@@ -38,6 +43,7 @@ usernamePasswordCtrlJs showURL = [jmacro|
             if (datum.jrStatus == "Ok") {
               $scope.username_password_error = '';
               userService.updateFromToken(datum.jrData.token);
+              `loginRedirect postLoginRedirect`
             } else {
               userService.clearUser();
               $scope.username_password_error = datum.jrData;
@@ -299,3 +305,12 @@ usernamePasswordCtrlJs showURL = [jmacro|
 
   }
   |]
+  where
+    loginRedirect (Just plr) =
+      [jmacro|
+             // console.log(`"loginRedirect - " ++ show plr`);
+             window.location.href = `plr`;
+             |]
+    loginRedirect Nothing =
+      BlockStat []
+
